@@ -47,8 +47,8 @@ import rednotebook.util.unicode
 from rednotebook.gui.menu import MainMenuBar
 from rednotebook.external.htmltextview import HtmlWindow
 from rednotebook.gui.options import OptionsManager
-from rednotebook.gui import widgets
-from rednotebook.gui.widgets import CustomComboBoxEntry, CustomListView
+from rednotebook.gui import customwidgets
+from rednotebook.gui.customwidgets import CustomComboBoxEntry, CustomListView
 from rednotebook.gui.richtext import HtmlEditor
 from rednotebook.util import filesystem
 from rednotebook import info
@@ -62,6 +62,10 @@ from rednotebook.gui import categories
 from rednotebook.gui import t2t_highlight
 from rednotebook.gui import browser
 
+test_zeitgeist = False
+if test_zeitgeist:
+	from rednotebook.gui import journalgeist
+
 
 class MainWindow(object):
 	'''
@@ -125,10 +129,9 @@ class MainWindow(object):
 		self.editPane = self.builder.get_object('editPane')
 		
 		# Only add the config variable if webkit is available
-		#TODO: Change to 1 when tested
-		use_webkit = browser.webkit and self.redNotebook.config.read('useWebkit', 0)
+		use_webkit = browser.webkit and self.redNotebook.config.read('useWebkit', 1)
 		
-		logging.info('Using webkit: %s' % use_webkit)
+		logging.info('Using webkit for previews: %s' % use_webkit)
 		
 		if use_webkit:
 			self.html_editor = browser.HtmlView()
@@ -165,6 +168,7 @@ class MainWindow(object):
 			'on_previewButton_clicked': self.on_previewButton_clicked,
 			
 			'on_mainFrame_configure_event': self.on_mainFrame_configure_event,
+			'on_mainFrame_window_state_event': self.on_mainFrame_window_state_event,
 			
 			'on_addNewEntryButton_clicked': self.on_addNewEntryButton_clicked,
 			'on_addTagButton_clicked': self.on_addTagButton_clicked,
@@ -209,6 +213,23 @@ class MainWindow(object):
 		self.set_tooltips()
 		
 		
+		# Only add the config variable if zeitgeist is available
+		use_zeitgeist = test_zeitgeist and journalgeist.zeitgeist and \
+						self.redNotebook.config.read('useZeitgeist', 0)
+		logging.info('Using zeitgeist: %s' % use_zeitgeist)
+		
+		if use_zeitgeist:
+			self.setup_zeitgeist_view()
+		
+	
+	def setup_zeitgeist_view(self):
+		'''Zeigeist integration'''
+		#from rednotebook.gui.journalgeist import JournalZeitgeistWidget
+		zeitgeist_view = journalgeist.JournalZeitgeistWidget(self.redNotebook.actualDate)
+		right_vbox = self.builder.get_object('right_vbox')
+		right_vbox.pack_start(zeitgeist_view)
+		
+		
 	def set_tooltips(self):
 		'''
 		Little work-around:
@@ -256,7 +277,7 @@ class MainWindow(object):
 	# TRAY-ICON / CLOSE --------------------------------------------------------
 			
 	def setup_tray_icon(self):
-		self.tray_icon = widgets.RedNotebookTrayIcon()
+		self.tray_icon = customwidgets.RedNotebookTrayIcon()
 		visible = (self.redNotebook.config.read('closeToTray', 0) == 1)
 		self.tray_icon.set_visible(visible)
 		logging.debug('Tray icon visible: %s' % visible)
@@ -317,14 +338,14 @@ class MainWindow(object):
 		'''
 		Exit if not closeToTray
 		'''
-		#logging.debug('on_mainFrame_destroy')
+		logging.debug('Main frame destroyed')
 		#self.saveToDisk(exitImminent=False)
 		
 		if self.redNotebook.config.read('closeToTray', 0):
 			self.hide()
 		
 			# the default handler is _not_ to be called, 
-			# and therefore the window will not be destroyed. 
+			# and therefore the window will not be destroyed.
 			return True
 		else:
 			self.redNotebook.exit()
@@ -411,9 +432,13 @@ class MainWindow(object):
 		
 			
 	def on_searchNotebook_switch_page(self, notebook, page, pageNumber):
-		#if pageNumber == 0:
+		if pageNumber == 0:
 			# Switched to search tab
 			#self.searchTreeView.update_data()
+			
+			# Put cursor into search field, when search tab is opened
+			# Strangely it only works with the idle_add workaround
+			gobject.idle_add(self.searchBox.entry.grab_focus)
 		if pageNumber == 1:
 			# Switched to cloud tab
 			self.cloud.update(force_update=True)
@@ -422,7 +447,16 @@ class MainWindow(object):
 	def setup_clouds(self):
 		self.cloudBox = self.builder.get_object('cloudBox')
 		
-		self.cloud = CloudView(self.redNotebook)
+		if browser.webkit:
+			from rednotebook.gui.clouds import Cloud
+			self.cloud = Cloud(self.redNotebook)
+			logging.info('Using pywebkitgtk for the clouds.')
+		else:
+			logging.warning('pywebkitgtk is not installed and cannot ' \
+					'be used for the clouds. You may experience errors')
+			#self.searchNotebook.remove_page(1)
+			self.cloud = CloudView(self.redNotebook)
+			
 		self.cloudBox.pack_start(self.cloud)
 		
 		self.cloudComboBox = self.builder.get_object('cloudComboBox')
@@ -444,6 +478,20 @@ class MainWindow(object):
 		self.redNotebook.config['mainFrameWidth'] = mainFrameWidth
 		self.redNotebook.config['mainFrameHeight'] = mainFrameHeight
 		
+	def on_mainFrame_window_state_event(self, widget, event):
+		'''
+		The "window-state-event" signal is emitted when window state 
+		of widget changes. For example, for a toplevel window this 
+		event is signaled when the window is iconified, deiconified, 
+		minimized, maximized, made sticky, made not sticky, shaded or 
+		unshaded.	
+		'''
+		state = event.new_window_state
+		maximized = state and gtk.gdk.WINDOW_STATE_MAXIMIZED
+		maximized = int(bool(maximized))
+		self.redNotebook.config['mainFrameMaximized'] = maximized
+		
+		
 	def on_backOneDayButton_clicked(self, widget):
 		self.redNotebook.goToPrevDay()
 		
@@ -499,6 +547,7 @@ class MainWindow(object):
 			self.redNotebook.open_journal(default_dir, load_files=True)
 			### Translators: The default journal is located at $HOME/.rednotebook/data
 			self.redNotebook.showMessage(_('The default journal has been opened'))
+			logging.info('The default journal has been opened')
 			
 	def show_save_error_dialog(self, exitImminent):
 		dialog = self.builder.get_object('save_error_dialog')
@@ -530,6 +579,9 @@ class MainWindow(object):
 				self.builder.get_object('mainPane').get_position()
 		config['rightDividerPosition'] = \
 				self.builder.get_object('editPane').get_position()
+				
+		# Remember if window was maximized
+		
 		
 		# Actually this is unnecessary as the list gets saved when it changes
 		# so we use it to sort the list ;)
@@ -550,7 +602,8 @@ class MainWindow(object):
 		
 		self.mainFrame.resize(mainFrameWidth, mainFrameHeight)
 		
-		#self.mainFrame.maximize()
+		if config.read('mainFrameMaximized', 0):
+			self.mainFrame.maximize()
 		
 		if config.has_key('leftDividerPosition'):
 			self.builder.get_object('mainPane').set_position(config.read('leftDividerPosition', -1))
@@ -602,7 +655,7 @@ class MainWindow(object):
 			<menuitem action="Bold"/>
 			<menuitem action="Italic"/>
 			<menuitem action="Underline"/>
-			<menuitem action="Stricken"/>
+			<menuitem action="Strikethrough"/>
 		</popup>
 		</ui>'''
 			
@@ -615,13 +668,11 @@ class MainWindow(object):
 		def tmpl(word):
 			return word + ' (Ctrl+%s)' % word[0]
 		
-		def apply_format(action):
+		def apply_format(action, format='bold'):
 			format_to_markup = {'bold': '**', 'italic': '//', 'underline': '__',
-								'stricken': '--'}
+								'strikethrough': '--'}
 			if type(action) == gtk.Action:
 				format = action.get_name().lower()
-			else:
-				format = 'bold'
 			
 			markup = format_to_markup[format]
 			
@@ -647,8 +698,8 @@ class MainWindow(object):
 				apply_format,
 				)
 		# Create actions
-		strike_action = ('Stricken', gtk.STOCK_STRIKETHROUGH, \
-				_('Stricken'), None, None, apply_format,)
+		strike_action = ('Strikethrough', gtk.STOCK_STRIKETHROUGH, \
+				_('Strikethrough'), None, None, apply_format,)
 		actions = [get_action(format, trans) for format, trans in \
 					(('Bold', _('Bold')), \
 					('Italic', _('Italic')), \
@@ -831,6 +882,12 @@ class MainWindow(object):
 		if response == gtk.RESPONSE_OK:
 			dirs.last_pic_dir = picture_chooser.get_current_folder()
 			base, ext = os.path.splitext(picture_chooser.get_filename())
+			
+			# On windows firefox accepts absolute filenames only 
+			# with the file:// prefix
+			if not base.startswith('file://'):
+				base = 'file://' + base
+				
 			self.dayTextField.insert('[""%s""%s]' % (base, ext))
 			
 	def on_insert_file_menu_item_activate(self, widget):
@@ -951,6 +1008,15 @@ class MainWindow(object):
 			#Ask at startup
 			self.redNotebook.config['checkForNewVersion'] = 1
 			
+	def highlight_text(self, search_text):
+		# let the search function highlight found strings in the page
+		#if self.preview_mode:
+			# If we search twice on the same day, the html is not reloaded
+			# so we have to manually highlight
+		self.html_editor.highlight(search_text)
+		#else:
+		self.dayTextField.highlight(search_text)
+			
 
 class NewEntryDialog(object):
 	def __init__(self, mainFrame):
@@ -1068,7 +1134,6 @@ class SearchComboBox(CustomComboBoxEntry):
 		"""
 			Called when the entry changes
 		"""
-		#self.search(entry.get_text())
 		self.search(self.get_active_text())
 		
 	def on_entry_activated(self, entry):
@@ -1086,6 +1151,13 @@ class SearchComboBox(CustomComboBoxEntry):
 		self.search(self.get_active_text())
 			
 	def search(self, searchText):
+		if self.searchType == 0:
+			# Tell the webview which text to highlight after the html is loaded
+			self.mainWindow.html_editor.search_text = searchText
+		
+			# Highlight all occurences in the current day's text
+			self.mainWindow.highlight_text(searchText)
+		
 		self.mainWindow.searchTreeView.update_data(searchText)
 		
 		
@@ -1120,6 +1192,7 @@ class CloudView(HtmlWindow):
 		self.ignore_list = map(str.lower, self.ignore_list)
 		logging.info('Cloud ignore list: %s' % self.ignore_list)
 		
+		### Translators: These are example whitelist words with 4 or less letters
 		default_include_list = _('mtv, spam, work, job, play')
 		self.include_list = config.read_list('cloudIncludeList', default_include_list)
 		self.include_list = map(str.lower, self.include_list)
@@ -1270,7 +1343,7 @@ class SearchTreeView(object):
 		self.treeView.connect('row_activated', self.on_row_activated)
 		
 		
-	def update_data(self, searchText=''):
+	def update_data(self, searchText=''):		
 		self.treeStore.clear()
 		
 		rows = None
@@ -1308,11 +1381,7 @@ class SearchTreeView(object):
 		self.redNotebook.changeDate(newDate)
 		
 		if self.searchType == 0:
-			# let the search function highlight found strings in the page
-			if self.mainWindow.preview_mode:
-				self.mainWindow.html_editor.highlight(self.searched_text)
-			else:
-				self.mainWindow.dayTextField.highlight(self.searched_text)
+			self.mainWindow.highlight_text(self.searched_text)
 		
 		
 	def set_search_type(self, searchType):
@@ -1344,7 +1413,7 @@ class DayTextField(object):
 		
 		# Enable drag&drop
 		#self.dayTextView.connect('drag-drop', self.on_drop) # unneeded
-		#self.dayTextView.connect('drag-data-received', self.on_drag_data_received)
+		self.dayTextView.connect('drag-data-received', self.on_drag_data_received)
 		
 	def set_text(self, text, undoing=False):
 		self.insert(text, overwrite=True, undoing=undoing)
@@ -1390,9 +1459,15 @@ class DayTextField(object):
 								#| gtk.SEARCH_CASE_INSENSITIVE # non-existent
 								)
 			
-			# When we find one variant, highlight it and quit
+			# When we find one variant, highlight it, scroll to it and quit
 			if iter_tuple:
 				self.set_selection(*iter_tuple)
+				
+				# It is safer to scroll to a mark than an iter
+				mark = self.dayTextBuffer.create_mark('highlight', iter_tuple[0], left_gravity=False)
+				#self.dayTextView.scroll_to_iter(iter_tuple[0], 0)
+				self.dayTextView.scroll_to_mark(mark, 0)
+				self.dayTextBuffer.delete_mark(mark)
 				return
 		
 			
@@ -1547,7 +1622,7 @@ class DayTextField(object):
 			return ext.lower().strip('.') in 'png jpeg jpg gif eps bmp'.split()
 		
 		uris = selection.data.strip('\r\n\x00')
-		print 'URI', uris
+		logging.debug('URIs: "%s"' % uris)
 		uris = uris.split() # we may have more than one file dropped
 		uris = map(str.strip, uris)
 		for uri in uris:
@@ -1574,9 +1649,16 @@ class Statusbar(object):
 		self.lastMessageID = None
 		self.timespan = 7
 		
+	def remove_message(self):
+		if hasattr(self.statusbar, 'remove_message'):
+			self.statusbar.remove_message(self.contextID, self.lastMessageID)
+		else:
+			# Deprecated
+			self.statusbar.remove(self.contextID, self.lastMessageID)
+		
 	def showText(self, text, error=False, countdown=True):
 		if self.lastMessageID is not None:
-			self.statusbar.remove(self.contextID, self.lastMessageID)
+			self.remove_message()
 		self.lastMessageID = self.statusbar.push(self.contextID, text)
 		
 		self.error = error
@@ -1613,6 +1695,10 @@ class Calendar(object):
 		self.redNotebook = redNotebook
 		self.calendar = calendar
 		
+		week_numbers = self.redNotebook.config.read('weekNumbers', 0)
+		if week_numbers:
+			calendar.set_property('show-week-numbers', True)
+		
 		self.date_listener = self.calendar.connect('day-selected', self.on_day_selected)
 		
 	def on_day_selected(self, cal):
