@@ -21,18 +21,69 @@ from __future__ import with_statement
 
 import os
 import sys
-import mimetypes
 import logging
+import time
+import codecs
 
 import gtk
 
 
 from rednotebook.util import filesystem
 
+
+
+example_text = '''\
+=== This is a template ===
+
+To edit it, click the arrow right of "Template" \
+and select the template under "Edit Template".
+
+Templates can contain any formatting or content that is also \
+allowed in normal entries.
+
+Your text can be:
+
+- **bold**
+- //italic//
+- __underlined__
+- --strikethrough--
+
+
+You can add images to your template:
+
+**Images:** [""/path/to/your/picture"".jpg]
+
+You can link to almost everything:
+
+- **links to files on your computer:** [filename.txt ""/path/to/filename.txt""]
+- **links to directories:** [directory name ""/path/to/directory/""]
+- **links to websites:** [RedNotebook Homepage ""http://rednotebook.sourceforge.net""]
+
+
+As you see, **bullet lists** are also available. As always you have to add two \
+empty lines to end a list.
+
+Additionally you can have **titles** and **horizontal lines**:
+
+===Title===
+
+==================== 
+
+**Macros**:
+
+When a template is inserted, every occurence of "$date$" is converted to \
+the current date. You can set the date format in the preferences.
+'''
+
+
+
+
 class TemplateManager(object):
 	def __init__(self, mainWindow):
 		self.mainWindow = mainWindow
 		
+		self.dirs = mainWindow.redNotebook.dirs
+		
 		self.merge_id = None
 		self.actiongroup = None
 		
@@ -40,11 +91,14 @@ class TemplateManager(object):
 	def on_insert(self, action):
 		title = action.get_name()
 		
+		# strip 'Insert'
+		title = title[6:]
+		
 		if title == 'Weekday':
 			text = self.get_weekday_text()
 		else:
 			text = self.get_text(title)
-		self.mainWindow.dayTextField.insert_template(text)
+		self.mainWindow.dayTextField.insert(text)
 		
 		
 	def on_edit(self, action):
@@ -63,7 +117,7 @@ class TemplateManager(object):
 		filesystem.open_url(filename)
 		
 	def on_new_template(self, action):
-		dialog = gtk.Dialog('Choose Template Name')
+		dialog = gtk.Dialog(_('Choose Template Name'))
 		dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
 		dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
 		
@@ -72,46 +126,13 @@ class TemplateManager(object):
 		dialog.get_content_area().pack_start(entry)
 		dialog.show_all()
 		response = dialog.run()
-		dialog.hide()
-		
-		example_text = '''\
-This is a template file. It can contain any formatting or content that is also \
-allowed in normal entries.
-
-Your text can be:
-
-- **bold**
-- //italic//
-- __underlined__
-- --stricken--
-
-
-You can add images to your template:
-
-**images:** [""/path/to/your/picture"".jpg]
-
-You can link to almost everything:
-
-- **links to files on your computer:** [filename.txt ""/path/to/filename.txt""]
-- **links to directories:** [directory name ""/path/to/directory/""]
-- **links to websites:** [RedNotebook Homepage ""http://rednotebook.sourceforge.net""]
-
-
-As you see, **bullet lists** are also available. As always you have to add two \
-empty lines to the end of a list.
-
-Additionally you can have **titles** and **horizontal lines**:
-
-===Title===
-
-==================== 
-		'''
+		dialog.hide()		
 		
 		if response == gtk.RESPONSE_OK:
 			title = entry.get_text()
 			if not title.lower().endswith('.txt'):
 				title += '.txt'
-			filename = os.path.join(filesystem.templateDir, title)
+			filename = os.path.join(self.dirs.templateDir, title)
 			
 			filesystem.makeFile(filename, example_text)
 			
@@ -119,20 +140,39 @@ Additionally you can have **titles** and **horizontal lines**:
 			
 	
 	def on_open_template_dir(self):
-		filesystem.open_url(filesystem.templateDir)
+		filesystem.open_url(self.dirs.templateDir)
+		
+	
+	def getTemplateFile(self, basename):
+		return os.path.join(self.dirs.templateDir, str(basename) + '.txt')
 		
 		
 	def get_text(self, title):
 		filename = self.titles_to_files.get(title, None)
 		if not filename:
 			return ''
+			
+		text = filesystem.read_file(filename)
 		
+		# An Error occured
+		if not text:
+			text = \
+'''This template contains no text or has unreadable content \
+(the template file's encoding must be utf-8). \
+To edit it, click the arrow right of "Template" \
+and select the template under "Edit Template".'''
+			
+		# convert every "$date$" to the current date
+		default_date_string = '%A, %x %X'
+		date_string = self.mainWindow.redNotebook.config.read('dateTimeString', default_date_string)
+		date = time.strftime(date_string)
 		try:
-			with open(filename, 'r') as templateFile:
-				 text = templateFile.read()
-		except IOError, Error:
-			logging.error('Template File %s not found' % name)
-			text = ''
+			text = text.replace('$date$', date)
+		except UnicodeDecodeError, err:
+			# TODO: Figure out real source of the error,
+			# maybe load config with filesystem.read_file
+			logging.error('Error replacing $date$: "%s"' % err)
+		
 		return text
 		
 		
@@ -144,7 +184,7 @@ Additionally you can have **titles** and **horizontal lines**:
 		
 	
 	def get_available_template_files(self):
-		dir = filesystem.templateDir
+		dir = self.dirs.templateDir
 		files = os.listdir(dir)
 		files = map(lambda basename: os.path.join(dir, basename), files)
 		
@@ -154,12 +194,6 @@ Additionally you can have **titles** and **horizontal lines**:
 		# No tempfiles
 		files = filter(lambda file: not file.endswith('~'), files)
 		
-		#textfiles = []
-		#for file in files:
-		#	type, encoding = mimetypes.guess_type(file)
-		#	if type == 'text/plain':
-		#		textfiles.append(file)
-		#print 'TPL Files', files
 		return files
 	
 	
@@ -184,8 +218,27 @@ Additionally you can have **titles** and **horizontal lines**:
 		menu_xml = '''\
 		<ui>
 		<popup action="TemplateMenu">
+		'''
+		
+		insert_menu_xml = '''\
+			<menu action="InsertMenu">
+				<menuitem action="InsertWeekday"/>
+				<separator name="sep4"/>
+		'''
+		for title in sorted_titles:
+			if title not in map(str, range(1,8)):
+				insert_menu_xml += '''\
+				<menuitem action="Insert%s"/>
+				''' % title
+		insert_menu_xml += '''\
+			</menu>
+		'''
+		
+		menu_xml += insert_menu_xml
+		
+		menu_xml += '''\
+			<separator name="sep5"/>
 			<menuitem action="NewTemplate"/>
-			<menuitem action="OpenTemplateDirectory"/>
 		'''
 			
 		edit_menu_xml = '''\
@@ -193,30 +246,19 @@ Additionally you can have **titles** and **horizontal lines**:
 				<menuitem action="EditWeekday"/>
 				<separator name="sep3"/>
 		'''
-		
 		for title in sorted_titles:
 			if title not in map(str, range(1,8)):
 				edit_menu_xml += '''\
 				<menuitem action="Edit%s"/>
 				''' % title
-				
 		edit_menu_xml += '''\
 			</menu>
 		'''
+		
 		menu_xml += edit_menu_xml
 		
-		menu_xml += '''\
-			<separator name="sep3"/>
-			<menuitem action="Weekday"/>
-		'''
-		
-		for title in sorted_titles:
-			if title not in map(str, range(1,8)):
-				menu_xml += '''\
-			<menuitem action="%s"/>
-				''' % title
-		
-		menu_xml += '''\
+		menu_xml +='''\
+			<menuitem action="OpenTemplateDirectory"/>
 		</popup>
 		</ui>'''
 			
@@ -233,26 +275,29 @@ Additionally you can have **titles** and **horizontal lines**:
 		actions = []			
 		
 		for title in sorted_titles:
-			insert_action = (title, None, title, None, None, \
+			insert_action = ('Insert' + title, None, title, None, None, \
 					lambda widget: self.on_insert(widget))
 			actions.append(insert_action)
 			edit_action = ('Edit' + title, None, title, None, None, \
 					lambda widget: self.on_edit(widget))
 			actions.append(edit_action)
 			
-		actions.append(('Weekday', gtk.STOCK_HOME, "This Weekday's Template", None, None, \
+		actions.append(('InsertWeekday', gtk.STOCK_HOME, _("This Weekday's Template"), None, None, \
 					lambda widget: self.on_insert(widget)))
 		
-		actions.append(('EditMenu', gtk.STOCK_EDIT, 'Edit Template', None, None, \
+		actions.append(('EditMenu', gtk.STOCK_EDIT, _('Edit Template'), None, None, \
 					None))
 		
-		actions.append(('EditWeekday', gtk.STOCK_HOME, 'This Weekday', None, None, \
+		actions.append(('InsertMenu', gtk.STOCK_ADD, _('Insert Template'), None, None, \
+					None))
+		
+		actions.append(('EditWeekday', gtk.STOCK_HOME, _("This Weekday's Template"), None, None, \
 					lambda widget: self.on_edit(widget)))
 		
-		actions.append(('NewTemplate', gtk.STOCK_NEW, 'Create New Template', None, None, \
+		actions.append(('NewTemplate', gtk.STOCK_NEW, _('Create New Template'), None, None, \
 					lambda widget: self.on_new_template(widget)))
 		
-		actions.append(('OpenTemplateDirectory', gtk.STOCK_DIRECTORY, 'Open Template Directory', None, None, \
+		actions.append(('OpenTemplateDirectory', gtk.STOCK_DIRECTORY, _('Open Template Directory'), None, None, \
 					lambda widget: self.on_open_template_dir()))
 		
 		
@@ -279,19 +324,20 @@ Additionally you can have **titles** and **horizontal lines**:
 	
 	def make_empty_template_files(self):
 		def getInstruction(dayNumber):
-			file = filesystem.getTemplateFile(dayNumber)
-			text = '''\
-The template for this weekday has not been edited. 
-If you want to have some text that you can add to that day every week, \
-edit the file [%s ""%s""] in a text editor.
-
-To do so, you can switch to "Preview" and click on the link to that file.
-			''' % (os.path.basename(file), file)
+			file = self.getTemplateFile(dayNumber)
+			#text = '''\
+#The template for this weekday has not been edited. 
+#If you want to have some text that you can add to that day every week, \
+#edit the file [%s ""%s""] in a text editor.
+#
+#To do so, you can switch to "Preview" and click on the link to that file.
+#			''' % (os.path.basename(file), file)
+			text = example_text
 			return text
 					
 		fileContentPairs = []
 		for dayNumber in range(1, 8):
-			fileContentPairs.append((filesystem.getTemplateFile(dayNumber), getInstruction(dayNumber)))
+			fileContentPairs.append((self.getTemplateFile(dayNumber), getInstruction(dayNumber)))
 		
 		template_help_text = '''\
 Besides templates for weekdays you can also have arbitrary named templates. 
@@ -309,13 +355,14 @@ You can switch to "Preview" mode and click on the link to get to the \
 If you come up with templates that could be useful for other people as well, \
 I would appreciate if you sent me your template file, so others can benefit \
 from it.
-		''' % (filesystem.templateDir, filesystem.templateDir)
+		''' % (self.dirs.templateDir, self.dirs.templateDir)
 		
-		template_help_filename = filesystem.getTemplateFile('Help')
+		template_help_filename = self.getTemplateFile('Help')
 		fileContentPairs.append((template_help_filename, template_help_text))
 		
-		# Only add the example templates the first time
-		if not self.mainWindow.redNotebook.firstTimeExecution:
+		# Only add the example templates the first time and just restore
+		# the day templates everytime
+		if not self.mainWindow.redNotebook.dirs.is_first_start:
 			filesystem.makeFiles(fileContentPairs)
 			return
 		
@@ -355,7 +402,7 @@ Additional items: Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx. Xxxxxxxxxxxxxxxxxxxxx
  - Xxxxxxxxxx Xxxxxxxxxxxx
 		'''
 		
-		template_meeting_filename = filesystem.getTemplateFile('Meeting')
+		template_meeting_filename = self.getTemplateFile('Meeting')
 		fileContentPairs.append((template_meeting_filename, template_meeting_text))
 		
 		
@@ -373,7 +420,7 @@ First we went to xxxxx then we got to yyyyy ...
 **Pictures:** [Image folder ""/path/to/the/images/""]
 		'''
 		
-		template_journey_filename = filesystem.getTemplateFile('Journey')
+		template_journey_filename = self.getTemplateFile('Journey')
 		fileContentPairs.append((template_journey_filename, template_journey_text))
 		
 		
